<?php

namespace AltSolution\Admin\Form\Component;

use AltSolution\Admin\Form\AbstractComponent;
use Illuminate\Support\Arr;

class FormSubmit extends AbstractComponent
{
    protected $view = 'form_submit';
    protected $enumerable = false;

    protected function customViewData()
    {
        $buttons = $this->getButtons();
        return compact('buttons');
    }

    protected function getButtons()
    {
        $optButtons = $this->getOption('buttons', ['save', 'save_apply']);
        if (!is_array($optButtons)) {
            throw new \Exception('Buttons must be defined as array');
        }
        $buttons = [];
        foreach ($optButtons as $optBtnKey => $optBtnValue) {
            if (is_array($optBtnValue)) {
                // [[..], [..]]
                $attributes = [];
                $type = Arr::get($optBtnValue, 'type');
                if ($type !== null) {
                    if (!in_array($type, ['button', 'reset', 'submit'])) {
                        throw new \Exception('Invalid button type: ' . $type);
                    }
                    $attributes['type'] = $type;
                }
                $name = Arr::get($optBtnValue, 'name');
                if ($name !== null) {
                    $attributes['name'] = $name;
                }
                $value = Arr::get($optBtnValue, 'value');
                if ($value !== null) {
                    $attributes['value'] = $value;
                }
                $color = Arr::get($optBtnValue, 'color');
                if ($color !== null) {
                    if (!in_array($color, ['primary', 'secondary', 'success', 'danger', 'warning', 'info', 'light', 'dark'/*, 'link'*/])) {
                        throw new \Exception('Invalid button color: ' . $color);
                    }
                }
                $button = [
                    'icon' => Arr::get($optBtnValue, 'icon'),
                    'text' => Arr::get($optBtnValue, 'text'),
                    'color' => $color ?: 'primary',
                    'attributes' => $attributes,
                ];
                $buttons[] = $button;
            } elseif (is_string($optBtnKey)) {
                // ['fa-icon' => 'Button text']
                $buttons[] = [
                    'icon' => $optBtnKey,
                    'text' => $optBtnValue,
                    'color' => 'primary',
                    'attributes' => [
                        'type' => 'submit',
                    ],
                ];
            } else {
                // predefined buttons
                // ['save', 'apply']
                switch ($optBtnValue) {
                    case 'save':
                        $button = [
                            'icon' => 'fas fa-save',
                            'text' => trans('admin::common.save'),
                            'color' => 'primary',
                            'attributes' => [
                                'type' => 'submit',
                            ],
                        ];
                        break;
                    // TODO: weak
                    case 'save_apply':
                        $button = [
                            'icon' => 'fas fa-check',
                            'text' => trans('admin::common.apply'),
                            'color' => 'primary',
                            'attributes' => [
                                'type' => 'submit',
                                'name' => 'button_apply',
                                'value' => '1',
                            ],
                        ];
                        break;
                    case 'apply':
                        $button = [
                            'icon' => 'fas fa-check',
                            'text' => trans('admin::common.apply'),
                            'color' => 'primary',
                            'attributes' => [
                                'type' => 'submit',
                            ],
                        ];
                        break;
                    case 'reset':
                        $button = [
                            'icon' => 'fas fa-sync', // fa-undo
                            'text' => trans('admin::common.reset'),
                            'color' => 'primary',
                            'attributes' => [
                                'type' => 'reset',
                            ],
                        ];
                        break;
                    case 'send':
                        $button = [
                            'icon' => 'fas fa-paper-plane',
                            'text' => trans('admin::common.send'),
                            'color' => 'primary',
                            'attributes' => [
                                'type' => 'submit',
                            ],
                        ];
                        break;
                    case 'exec':
                        $button = [
                            'icon' => 'fas fa-play',
                            'text' => trans('admin::common.exec_btn'),
                            'color' => 'primary',
                            'attributes' => [
                                'type' => 'submit',
                            ],
                        ];
                        break;
                    default:
                        throw new \Exception('Invalid predefined button: ' . $optBtnValue);
                }
                $buttons[] = $button;
            }
        }

        return $buttons;
    }
}
