<?php

namespace AltSolution\Admin\Form\Field;

use Exception;
use Generator;
use Illuminate\Database\Eloquent\Model;

class SelectModel extends Select
{
    protected function getChoices()
    {
        if (!isset($this->options['model'])) {
            throw new Exception('SelectModel field must have model');
        }

        $modelClass = $this->options['model'];
        /** @var Model $model */
        $model = new $modelClass;

        $pk = $this->getOption('primary_key', 'id');
        $tk = $this->getOption('title_key', 'title');
        $order = $this->getOption('order_by');

        return iterator_to_array($this->generateOptions($model, $pk, $tk, $order));
    }

    /**
     * @param Model $model
     * @param string|callable $pk
     * @param string|callable $tk
     * @param string|array $order
     * @return Generator
     */
    private function generateOptions($model, $pk, $tk, $order)
    {
        $query = $model->newQuery();
        if (is_string($order)) {
            $query->orderBy($order);
        } elseif (is_array($order)) {
            foreach ($order as $orderColumn => $orderDirection) {
                $query->orderBy($orderColumn, $orderDirection);
            }
        }

        $items = $query->get();

        if (is_callable($pk)) {
            $pkVal = function($item) use ($pk) { return $pk($item); };
        } else {
            $pkVal = function($item) use ($pk) { return $item[$pk]; };
        }
        if (is_callable($tk)) {
            $tkVal = function($item) use ($tk) { return $tk($item); };
        } else {
            $tkVal = function($item) use ($tk) { return $item[$tk]; };
        }

        foreach ($items as $item) {
            $key = $pkVal($item);
            $value = $tkVal($item);
            yield $key => $value;
        }
    }

}