<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Services\AclService;
use Illuminate\Http\Request;

class AclController extends Controller
{
    public function getAcl(AclService $aclService)
    {
        $this->authorize('permission', 'role.permission');

        $aclService->updatePermissions();
        $sections = $aclService->getPermissionsList();
        $roles = $aclService->getRoles();

        $form = cms_construct_form(function() {
            yield [
                'type' => 'form',
                'method' => 'post',
                'buttons' => ['save'],
            ];
        });
        $form->setLayout($form::LAYOUT_VERTICAL);

        $this->layout
            ->setMenuSection('users')
            ->setTitle(trans('admin::user.acl_permission'));
        return view('admin::users.acl', compact('roles', 'sections', 'form'));
    }

    public function postAcl(Request $request, AclService $aclService)
    {
        $this->authorize('permission', 'role.permission');

        $acl = $request->input('acl');

        $roles = $aclService->getRoles();
        foreach ($roles as $role) {
            $role->permissions()->detach();
            if (isset($acl[$role->id]) && is_array($acl[$role->id]) && count($acl[$role->id])) {
                $role->permissions()->attach($acl[$role->id]);
            }
        }

        $this->layout->addNotify('success', trans('admin::user.permission_saved'));

        return redirect()->route('admin::user_acl');
    }
}
