<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\EmailTemplate\TemplateRepository;
use AltSolution\Admin\Forms\EmailTemplateForm;
use AltSolution\Admin\Models\EmailTemplate;
use AltSolution\Admin\Services\EmailTemplateService;
use Illuminate\Http\Request;

class EmailTemplateController extends Controller
{
    public function index(EmailTemplateService $templateService)
    {
        $this->authorize('permission', 'email_template.list');

        $templateService->updateEmailTemplates();

        $items = EmailTemplate::query()
            ->paginate(config('admin.items_per_page', 20));

        $this->layout
            ->setMenuSection('email_template')
            ->setTitle(trans('admin::email_template.title'));
        return view('admin::email_template.list', compact('items'));
    }

    public function edit($id = null)
    {
        $this->authorize('permission', 'email_template.edit');

        $emailTemplate = EmailTemplate::query()
            ->findOrFail($id);

        $name = $emailTemplate->name;
        $template = app(TemplateRepository::class)->findByName($name);
        if (is_null($template)) {
            abort(404, 'Template not found: ' . $name);
        }

        $form = cms_create_form(EmailTemplateForm::class, $emailTemplate, compact('template'));

        $this->layout
            ->setMenuSection('email_template')
            ->setTitle(trans($id ? 'admin::email_template.edit' : 'admin::email_template.add'))
            ->addBreadcrumb(trans('admin::email_template.title'), route('admin::email_template_list'));

        if ($template->isMultilingual()) {
            return view('admin::email_template.edit_ml', compact('form', 'template'));
        }

        return view('admin::email_template.edit', compact('form', 'template'));
    }

    public function save(Request $request)
    {
        $this->authorize('permission', 'email_template.edit');

        $name = $request->name;
        $template = app(TemplateRepository::class)->findByName($name);
        if (is_null($template)) {
            abort(404, 'Template not found: ' . $name);
        }

        $model = EmailTemplate::query()->firstOrNew(['id' => $request->id]);
        $form = cms_create_form(EmailTemplateForm::class, $model, compact('template'));

        $validData = $form->validate($request);

        $model->fill($validData);
        $model->to_admin = (bool)$request->to_admin;
        $model->to_d_admin = (bool)$request->to_d_admin;
        if ($template->isBodyEditable()) {
            $model->html = (bool)$request->html;
            $model->layout = (bool)$request->layout;
        }

        event('admin.model.email-template.saving', $model);
        $model->save();
        event('admin.model.email-template.saved', $model);

        $this->layout->addNotify('success', trans('admin::email_template.saved'));

        if ($request->button_apply) {
            return redirect()->route('admin::email_template_edit', ['id' => $model->id]);
        }

        return redirect()->route('admin::email_template_list');
    }
}
