<?php

namespace AltSolution\Admin\Http\Middleware;

use AltSolution\Admin\System\System;
use Closure;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class BootSystem
{
    use AuthorizesRequests;

    /**
     * @var System
     */
    private $system;

    /**
     * BootSystem constructor.
     * @param System $system
     */
    public function __construct(System $system)
    {
        $this->system = $system;
    }

    /**
     * Handle an incoming request.
     *
     * @param  Request  $request
     * @param  \Closure  $next
     * @param  string|null  $guard
     * @return mixed
     */
    public function handle($request, Closure $next, $guard = null)
    {
        $this->authorize('permission', 'admin');

        $this->bootSystem($request);

        $response = $next($request);

        $this->pullNotifications($request, $response);

        return $response;
    }

    /**
     * @param Request $request
     * @throws \AltSolution\Admin\System\Exception
     */
    private function bootSystem($request)
    {
        $backendUrl = cms_admin_url();
        if ($request->is($backendUrl, $backendUrl . '/*')) {
            app()->setLocale(cms_admin_locale());

            $this->system->setIsBackend();

            $modules = app()->tagged('admin.system.module');
            $this->system->registerModules($modules);
            $this->system->bootModules();
        }
    }

    /**
     * @param Request $request
     * @param Response $response
     */
    private function pullNotifications($request, $response)
    {
        $pullNotify = $request->header('X-CMS-PULL-NOTIFY');
        if ($pullNotify) {
            $notifications = $this->system->getLayout()->getNotifications()->pullAll();
            if ($notifications) {
                $notificationsJson = json_encode($notifications);
                // WARN: only if less then 2KB
                if (strlen($notificationsJson) <= 2048) {
                    $response->header('X-CMS-NOTIFY', $notificationsJson);
                }
            }
        }
    }
}
