<?php

namespace AltSolution\Admin\Modules;

use AltSolution\Admin\System\AbstractModule;
use AltSolution\Admin\System\MenuBuilder;
use AltSolution\Admin\System\PermissionBuilder;
use Illuminate\Routing\Router;

class MenuModule extends AbstractModule
{
    public function buildMenu(MenuBuilder $menu)
    {
        $menu->addSection(trans('admin::menu.ms_menu'), 'fas fa-bars', route('admin::menu_list'))
            ->setPermission('menu.list')
            ->setAlias('menu')
            ->setWeight(300);
    }

    public function buildPermissions(PermissionBuilder $builder)
    {
        $builder->addSection('menu', 'admin::menu.ps_menu');
        $builder->addPermission('menu.list', 'admin::menu.p_menu_list');
        $builder->addPermission('menu.edit', 'admin::menu.p_menu_edit');
        $builder->addPermission('menu.delete', 'admin::menu.p_menu_delete');
        $builder->addPermission('menuitem.list', 'admin::menu.p_menu_item_list');
        $builder->addPermission('menuitem.edit', 'admin::menu.p_menu_item_edit');
        $builder->addPermission('menuitem.delete', 'admin::menu.p_menu_item_delete');
    }

    public function registerRoutes(Router $router)
    {
        $controller = app('admin.controller.menu');
        $router->get('menu', [$controller, 'getIndex'])->name('menu_list');
        $router->get('menu/edit/{id?}', [$controller, 'getEdit'])->name('menu_edit');
        $router->post('menu/save', [$controller, 'postSave'])->name('menu_save');
        $router->post('menu/action', [$controller, 'action'])->name('menu_action');

        $controller = app('admin.controller.menu_item');
        $router->get('menu/items/{id}', [$controller, 'getIndex'])->name('menu_item_list');
        $router->get('menu/items/edit/{id}/{id2?}', [$controller, 'getEdit'])->name('menu_item_edit');
        $router->post('menu/items/save', [$controller, 'postSave'])->name('menu_item_save');
        $router->post('menu/items/action', [$controller, 'action'])->name('menu_item_action');
    }
}