<?php

namespace AltSolution\Admin\Rules;

use DOMDocument;
use Illuminate\Contracts\Validation\Rule;
use Symfony\Component\HttpFoundation\File;

class ValidSvg implements Rule
{
    /**
     * Determine if the validation rule passes.
     *
     * @param  string $attribute
     * @param  mixed $value
     * @return bool
     */
    public function passes($attribute, $value)
    {
        if (!$this->isValidFileInstance($value)) {
            return false;
        }

        $oldInternalErrors = libxml_use_internal_errors(true);
        $doc = new DOMDocument;
        $loaded = $doc->load($value->getRealPath());
        libxml_use_internal_errors($oldInternalErrors);
        if ($loaded === false) {
            return false;
        }

        $root = $doc->documentElement;
        if (!$root) {
            return false;
        }

        $ns = $root->getAttribute('xmlns');
        if (!in_array($ns, ['http://www.w3.org/2000/svg', 'https://www.w3.org/2000/svg'], true)) {
            return false;
        }

        return true;

        /*
        $xpath = new DOMXPath($doc);
        $xpath->registerNamespace('svg', 'http://www.w3.org/2000/svg');
        $xpath->query('svg:...');
        echo $doc->saveXML();
        */
    }

    /**
     * Get the validation error message.
     *
     * @return string
     */
    public function message()
    {
        return trans('admin::validation.svg');
    }

    /**
     * Check that the given value is a valid file instance.
     * @see \Illuminate\Validation\Concerns\ValidatesAttributes::isValidFileInstance
     *
     * @param  mixed  $value
     * @return bool
     *
     */
    private function isValidFileInstance($value)
    {
        if ($value instanceof File\UploadedFile && !$value->isValid()) {
            return false;
        }

        return $value instanceof File\File;
    }
}
