<?php

namespace AltSolution\Admin\Services;

use AltSolution\Admin\Models\AclPermission;
use AltSolution\Admin\Models\AclRole;
use Illuminate\Support\Arr;

final class AclService
{
    public function getRoles()
    {
        return AclRole::all();
    }

    /**
     * Add new permissions, remove not existing or deleted
     *
     * @return void
     */
    public function updatePermissions()
    {
        $allPermissions = cms_system()->getPermission()->getAllPermissions();
        $existsPermissions = AclPermission::all();
        $updatedPermissions = [];

        foreach ($allPermissions as $allPermission) {
            $existsPermission = $existsPermissions
                ->where('name', $allPermission->getName())
                ->first();

            if ($existsPermission === null) {
                $newPermission = new AclPermission();
                $newPermission->name = $allPermission->getName();
                $newPermission->description = $allPermission->getDescription();
                $newPermission->save();

                $updatedPermissions[] = $newPermission->id;
            } else {
                $existsPermission->description = $allPermission->getDescription();
                if ($existsPermission->isDirty()) {
                    $existsPermission->save();
                }

                $updatedPermissions[] = $existsPermission->id;
            }
        }

        foreach ($existsPermissions as $existsPermission) {
            if (!in_array($existsPermission->id, $updatedPermissions)) {
                $existsPermission->delete();
            }
        }
    }

    public function getPermissionsList()
    {
        $allPermissions = [];
        foreach (AclPermission::all() as $permission) {
            $allPermissions[$permission->name] = $permission;
        }

        $sections = [];
        foreach (cms_system()->getPermission()->getSections() as $section) {

            $sectionPermissions = [];
            foreach ($section->getPermissions() as $permission) {
                $aclPermission = Arr::get($allPermissions, $permission->getName());
                if ($aclPermission !== null) {
                    $sectionPermissions[] = $aclPermission;
                }
            }

            if (count($sectionPermissions)) {
                $sections[] = [
                    'name' => $section->getDescription(),
                    'permissions' => $sectionPermissions
                ];
            }
        }

        return $sections;
    }
}