<?php

namespace AltSolution\Admin\System;

class PermissionBuilder
{
    /**
     * @var PermissionSection[]
     */
    private $sections = [];
    /**
     * @var PermissionSection
     */
    private $currentSection;

    /**
     * @return PermissionSection|null
     */
    public function currentSection()
    {
        return $this->currentSection;
    }

    /**
     * @param string $name
     * @param string $description
     * @return PermissionSection
     */
    public function addSection($name, $description)
    {
        $section = new PermissionSection($name, $description);
        $this->sections[] = $section;
        $this->currentSection = $section;

        return $section;
    }

    /**
     * @param string $name
     * @return PermissionSection
     * @throws Exception
     */
    public function getSection($name)
    {
        $currentSection = null;
        foreach ($this->sections as $section) {
            if ($section->getName() == $name) {
                $currentSection = $section;
                break;
            }
        }
        if ($currentSection === null) {
            throw new Exception('Section not defined: ' . $name);
        }
        $this->currentSection = $currentSection;

        return $currentSection;
    }

    /**
     * @param string $name
     * @param string $description
     * @return PermissionItem
     * @throws \Exception
     */
    public function addPermission($name, $description)
    {
        if ($this->currentSection === null) {
            throw new \Exception('You must add section first');
        }

        $item = new PermissionItem($name, $description);
        $this->currentSection->addItem($item);

        return $item;
    }

    /**
     * @return PermissionSection[]
     */
    public function getSections()
    {
        return $this->sections;
    }

    /**
     * Perform section soring based on it weight
     */
    public function sort()
    {
        usort($this->sections, function(PermissionSection $a, PermissionSection $b) {
            if ($a->getWeight() == $b->getWeight()) {
                return 0;
            }
            return ($a->getWeight() > $b->getWeight()) ? -1 : 1;
        });
    }
}