<?php

namespace AltSolution\Admin\System;

class PermissionSection implements \IteratorAggregate, SortableInterface
{
    use SortableTrait;

    /**
     * @var string
     */
    private $name;
    /**
     * @var string
     */
    private $description;
    /**
     * @var PermissionItem[]
     */
    private $permissions = [];

    /**
     * PermissionSection constructor.
     * @param string $name
     * @param string $description
     */
    public function __construct($name, $description)
    {
        $this->name = $name;
        $this->description = $description;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param PermissionItem $permission
     */
    public function addItem(PermissionItem $permission)
    {
        $this->permissions[] = $permission;
    }

    public function getIterator()
    {
        return new \ArrayIterator($this->permissions);
    }

    /**
     * @return int
     */
    public function hasPermissions()
    {
        return count($this->permissions);
    }

    /**
     * @return PermissionItem[]
     */
    public function getPermissions()
    {
        return $this->permissions;
    }

}