<?php

namespace AltSolution\Admin\System;

use Illuminate\Support\Arr;

class SeoBuilder
{
    /**
     * @var SeoSection[]
     */
    private $sections = [];
    /**
     * @var SeoSection
     */
    private $currentSection;

    /**
     * @return SeoSection|null
     */
    public function currentSection()
    {
        return $this->currentSection;
    }

    public function addSection($name, $description)
    {
        $section = new SeoSection($name, $description);
        $this->sections[] = $section;
        $this->currentSection = $section;

        return $section;
    }

    /**
     * @param string $name
     * @return SeoSection
     * @throws Exception
     */
    public function getSection($name)
    {
        $currentSection = null;
        foreach ($this->sections as $section) {
            if ($section->getName() == $name) {
                $currentSection = $section;
                break;
            }
        }
        if ($currentSection === null) {
            throw new Exception('Section not defined: ' . $name);
        }
        $this->currentSection = $currentSection;

        return $currentSection;
    }

    public function addField($name, array $options = [])
    {
        if ($this->currentSection === null) {
            throw new \Exception('You must add or get section first');
        }

        $item = new SeoField($name, $options);
        $this->currentSection->addField($item);

        return $item;
    }

    public function addDefaultFields($help = null)
    {
        if ($this->currentSection === null) {
            throw new \Exception('You must add or get section first');
        }

        $fields = config('seo.default_fields');
        foreach ($fields as $name => $options) {
            if ($help !== null) {
                $options['help'] = $help;
            }
            $item = new SeoField($name, $options);
            $this->currentSection->addField($item);
        }
    }

    public function getSections()
    {
        return $this->sections;
    }

    public function sort()
    {
        usort($this->sections, function(SeoSection $a, SeoSection $b) {
            if ($a->getWeight() == $b->getWeight()) {
                return 0;
            }
            return ($a->getWeight() > $b->getWeight()) ? -1 : 1;
        });
    }
}